--[[

    Description:  
        The purpose of this script it to allow an administrator the ability to insert the appropriate SIP header entries
        into the SIP INVITE to RedSky to enable the alternate ID matching.  The script supports 3 configuration options. 
        The configuration choosen will be based on the parameters that are defined for the script on the SIP Trunk.

    Script Parameters:

    RedSky-CustomerID  --  This is the HELD ID for the organization in the RedSky admin portal.  

    Optional Parameters:

    Ext-Length -- 3-11 (recommended, but not enforced)
    Notes: Use this field to determine the length of a non-E164 extension to send to Redsky as the alternate ID.
    If the line DN has a format of 51234, then setting the value to 5 will insert the 5 digits into the E911-User-ID field.
    Use either Ext-Length or Agent-Ext, but not both.

    Agent-Ext  --  Prefix digits used to identify agent extensions
    Notes: define the leading digits exclsively used for agent extensions.  The prefix digits will be used to identify 
    agent lines vs DID lines.  If an agent line is defined as 1002001234, with the 100200 being the prefix for all 
    agent lines, then this field should have 100200 as the value.
    Use either Ext-Length or Agent-Ext, but not both.

    Alternate-Callback-Number --  10 digit E164 number for callbacks.
    Notes: This number will be passed as a callback number in addition to the original calling party identifier.
    This field can be used to route calls to a security desk number or front desk receptionist number if a callback is needed.
    
    NOTE: Adding an Alternate-Callback-Number will override the callback number set for the user in Redsky.

    Configure the following script parameters on the SIP Normalization Script in Unified Communications Manager (UCM)
        Memory Threshold -- 50 kilobytes
        Lua Instruction Threshold -- 1000 instructions
--]]

trace.enable()
M = {}

-- Allow this header to be passed through
M.allowHeaders = {"E911-Organization-ID", "E911-User-ID", "E911-Callback-Number"}

--Global variables

-- Read the  RedSky-CustomerID value configured on the SIP Trunk
rs_id = scriptParameters.getValue("RedSky-CustomerID")

-- Read the Alternate-Callback-Number value configured on the SIP Trunk
rs_callback = scriptParameters.getValue("Alternate-Callback-Number")

-- Read the RedSky-CustomerID value configured on the SIP Trunk
rs_ext_length = scriptParameters.getValue("Ext-Length")

-- Read the Agent-Ext value configured on the SIP Trunk
rs_agent_ext = scriptParameters.getValue("Agent-Ext")


function M.outbound_INVITE(msg)

    --Redsky always requires a customerID in the SIP INVITE check for existance.  
    --If the header already exists, then the script will assume the other Redsky headers exist and this UCM node should non alter the message, but pass it on    to the next hop.
    --If this script does not have the RedSky-CustomerID defined, then the script reports the error and does not alter the INVITE.
    --If the E911-Organization-ID does not already exist and there is a valid RedSky-CustomerID defined, then the script will insert the defined Redsky 
    --headers 

    local hdrexists = msg:getHeader("E911-Organization-ID")
    if hdrexists 
    then
        trace.format("SIP INVITE contains a valid E911-Organization-ID, no changes needed")
        return
    end

    if rs_id
    then
        --Insert the RedSky-CustomerID value into the INVITE
        msg:addHeader("E911-Organization-ID", rs_id)
        --trace.format("Inserted E911-Organization-ID: %s", rs_id)
        
        --Redsky supports an alternate callback number.  If parameters exists, add it to the SIP header.
        if rs_callback 
        then
            msg:addHeader("E911-Callback-Number", rs_callback)
            --trace.format("Inserted E911-Callback-Number: %s", rs_callback)
        end

        --Insert the alternate ID for user matching if defined
        if rs_ext_length or rs_agent_ext
        then
            local fromheader = msg:getUri("From")
		
            if fromheader 
            then
                local from = sipUtils.parseUri(fromheader)
                local fromUser = from:getUser()
                if rs_agent_ext 
                then
                    if(string.find(fromUser, rs_agent_ext)) 
                    then
                        msg:addHeader("E911-User-ID", fromUser)
                        --trace.format("Calling party not an E.164, inserting %s as the E911-User-ID", fromUser)
                    else
                        trace.format("No string match")
                    end
                elseif rs_ext_length 
                then
                    if (string.len(fromUser) == tonumber(rs_ext_length)) 
                    then
                        msg:addHeader("E911-User-ID", fromUser)
                        --trace.format("Calling party %d digits, inserting %s as the E911-User-ID", tonumber(rs_ext_length), fromUser)
                    else
                        trace.format("Extenstion length not matched")
                    end
                 end
             end	
         end
    end
end
	
return M
